@@ -8,14 +8,14 @@ import os
 import textwrap
 import time
 import unittest
-import pytest
-from io import StringIO
-
 from functools import partial
+from io import StringIO
 from sys import version_info
 from tempfile import NamedTemporaryFile
 
-from asteval import NameFinder, Interpreter, make_symbol_table
+import pytest
+
+from asteval import Interpreter, NameFinder, make_symbol_table
 
 HAS_NUMPY = False
 try:
@@ -26,8 +26,6 @@ except ImportError:
     HAS_NUMPY = False
 
 
-version_info = (version_info.major, version_info.minor)
-
 class TestCase(unittest.TestCase):
     """testing of asteval"""
     def setUp(self):
@@ -95,7 +93,6 @@ class TestCase(unittest.TestCase):
         if HAS_NUMPY:
             assert_allclose(tval, val, rtol=1.e-4, atol=1.e-4)
 
-
     # noinspection PyUnresolvedReferences
     def istrue(self, expr):
         """assert that an expression evaluates to True"""
@@ -135,12 +132,29 @@ class TestCase(unittest.TestCase):
 class TestEval(TestCase):
     """testing of asteval"""
 
+    def test_py3(self):
+        assert version_info.major > 2
+
     def test_dict_index(self):
         """dictionary indexing"""
         self.interp("a_dict = {'a': 1, 'b': 2, 'c': 3, 'd': 4}")
         self.istrue("a_dict['a'] == 1")
         self.istrue("a_dict['d'] == 4")
 
+    def test_dict_set_index(self):
+        """dictionary indexing"""
+        self.interp("a_dict = {'a': 1, 'b': 2, 'c': 3, 'd': 4}")
+        self.interp("a_dict['a'] = -4")
+        self.interp("a_dict['e'] = 73")
+
+        self.istrue("a_dict['a'] == -4")
+        self.istrue("a_dict['e'] == 73")
+
+        self.interp("b_dict = {}")
+        self.interp("keyname = 'a'")
+        self.interp("b_dict[keyname] = (1, -1, 'x')")
+        self.istrue("b_dict[keyname] ==  (1, -1, 'x')")
+
     def test_list_index(self):
         """list indexing"""
         self.interp("a_list = ['a', 'b', 'c', 'd', 'o']")
@@ -166,8 +180,8 @@ class TestEval(TestCase):
         """nd array indexing"""
         if HAS_NUMPY:
             self.interp("a_ndarray = 5*arange(20)")
-            assert(self.interp("a_ndarray[2]")  == 10)
-            assert(self.interp("a_ndarray[4]")  == 20)
+            assert(self.interp("a_ndarray[2]") == 10)
+            assert(self.interp("a_ndarray[4]") == 20)
 
     def test_ndarrayslice(self):
         """array slicing"""
@@ -177,6 +191,7 @@ class TestEval(TestCase):
             self.interp("y = arange(20).reshape(4, 5)")
             self.istrue("y[:,3]  == array([3, 8, 13, 18])")
             self.istrue("y[...,1]  == array([1, 6, 11, 16])")
+            self.istrue("y[1,:] == array([5, 6, 7, 8, 9])")
             self.interp("y[...,1] = array([2, 2, 2, 2])")
             self.istrue("y[1,:] == array([5, 2, 7, 8, 9])")
 
@@ -449,7 +464,7 @@ class TestEval(TestCase):
             failed = False
             # noinspection PyBroadException
             try:
-                self.interp(expr, show_errors=False)
+                self.interp(expr, show_errors=False, raise_errors=True)
             except:
                 failed = True
 
@@ -462,7 +477,7 @@ class TestEval(TestCase):
             failed = False
             # noinspection PyBroadException
             try:
-                self.interp(expr, show_errors=False)
+                self.interp(expr, show_errors=False, raise_errors=True)
             except:
                 failed = True
             self.assertTrue(failed)
@@ -474,7 +489,7 @@ class TestEval(TestCase):
             failed = False
             # noinspection PyBroadException
             try:
-                self.interp(expr, show_errors=False)
+                self.interp(expr, show_errors=False, raise_errors=True)
             except:  # RuntimeError:
                 failed = True
             self.assertTrue(failed)
@@ -498,7 +513,7 @@ class TestEval(TestCase):
             failed, errtype, errmsg = False, None, None
             # noinspection PyBroadException
             try:
-                self.interp(expr, show_errors=False)
+                self.interp(expr, show_errors=False, raise_errors=True)
             except:
                 failed = True
             self.assertTrue(failed)
@@ -613,7 +628,7 @@ class TestEval(TestCase):
             self.interp.error = []
             # noinspection PyBroadException
             try:
-                self.interp("%s= 2" % w, show_errors=False)
+                self.interp("%s= 2" % w, show_errors=False, raise_errors=True)
             except:
                 pass
 
@@ -806,8 +821,25 @@ class TestEval(TestCase):
         self.interp("o = fcn(1, x=2)")
         self.check_error('TypeError')
 
+    def test_kwargx(self):
+        """test passing and chaining in **kwargs"""
+        self.interp(textwrap.dedent("""
+            def inner(foo=None, bar=None):
+                return (foo, bar)
+
+            def outer(**kwargs):
+                return inner(**kwargs)
+            """))
+
+        ret = self.interp("inner(foo='a', bar=2)")
+        assert(ret == ('a', 2))
+        ret = self.interp("outer(foo='a', bar=7)")
+        assert(ret == ('a', 7))
+        ret = self.interp("outer(**dict(foo='b', bar=3))")
+        assert(ret == ('b', 3))
+
     def test_nested_functions(self):
-        setup="""
+        setup = """
         def a(x=10):
             if x > 5:
                 return 1
@@ -827,7 +859,6 @@ class TestEval(TestCase):
         self.isvalue('o1', 3.5)
         self.isvalue('o2', 1.5)
 
-
     def test_astdump(self):
         """test ast parsing and dumping"""
         astnode = self.interp.parse('x = 1')
@@ -857,7 +888,6 @@ class TestEval(TestCase):
         self.interp("1<<1001")
         self.check_error('RuntimeError')
 
-
     def test_safe_open(self):
         self.interp('open("foo1", "wb")')
         self.check_error('RuntimeError')
@@ -885,7 +915,7 @@ class TestEval(TestCase):
         self.check_error('RuntimeError')  # Safe, safe_pow() catches this
         self.interp(
             "x = ((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((1))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))")
-        if version_info == (3, 9):
+        if version_info.minor > 8:
             self.isvalue('x', 1)
             self.check_error(None)
         else:
@@ -950,9 +980,9 @@ class TestEval(TestCase):
             x2 = aeval.symtable['x2']
             x3 = aeval.symtable['x3']
 
-            assert_allclose(x1, 0.50,     rtol=0.001)
+            assert_allclose(x1, 0.50, rtol=0.001)
             assert_allclose(x2, 0.866025, rtol=0.001)
-            assert_allclose(x3, 1.00,     rtol=0.001)
+            assert_allclose(x3, 1.00, rtol=0.001)
 
     def test_readonly_symbols(self):
 
@@ -990,7 +1020,6 @@ class TestEval(TestCase):
         assert(aeval("x") == 21)
         assert(aeval("y") == 17)
 
-
         assert(aeval("abs(8)") == 8)
         assert(aeval("abs(-8)") == 8)
         aeval("def abs(x): return x*2")
@@ -1005,7 +1034,6 @@ class TestEval(TestCase):
         assert(aeval2("abs(8)") == 8)
         assert(aeval2("abs(-8)") == 8)
 
-
     def test_chained_compparisons(self):
         self.interp('a = 7')
         self.interp('b = 12')
@@ -1042,7 +1070,7 @@ class TestEval(TestCase):
         # uses the function's __name__ attribute. Partials don't have a
         # __name__ attribute, so we want to make sure that an AttributeError is
         # not raised.
-    
+
         result = aeval("sqrt(-1)")
         assert aeval.error.pop().exc == ValueError
 
@@ -1069,6 +1097,7 @@ class TestEval(TestCase):
         """))
         assert 4 == self.interp("func_w()")
 
+
 class TestCase2(unittest.TestCase):
     def test_stringio(self):
         """ test using stringio for output/errors """
@@ -1078,5 +1107,6 @@ class TestCase2(unittest.TestCase):
         intrep("print('out')")
         self.assertEqual(out.getvalue(), 'out\n')
 
+
 if __name__ == '__main__':
     pytest.main(['-v', '-x', '-s'])
